// Class: TicTacToeGUI
//
// Author: Alyce Brady
//
// Created on 18 May 2004
//
// License Information:
//   This class is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation.
//
//   This class is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

import java.awt.Color;

import javax.swing.JMenu;

import edu.kzoo.grid.BoundedGrid;
import edu.kzoo.grid.Grid;
import edu.kzoo.grid.Location;
import edu.kzoo.grid.gui.GridAppFrame;
import edu.kzoo.kgui.MinimalFileMenu;

/**
 *  Tic-Tac-Toe Game:<br>
 *
 *  An <code>TicTacToeGUI</code> object represents a window with
 *  control buttons and a display for tic-tac-toe game.
 *  Most of the functionality is provided in the GridAppFrame
 *  superclass.
 *
 *  @author Alyce Brady
 *  @version 18 May 2004
 **/
public class TicTacToeGUI extends GridAppFrame
{
    private String usualTitle;
    private int numRows;
    private Color usualBgColor;
    private Color whiteWonBgColor;
    private Color blackWonBgColor;

    private WhiteBlackGame game;

    /** Constructs a new TicTacToeGUI object.
     *    @param title         window title
     *    @param helpMenu      Help menu
     *    @param usualBgColor  grid background color during game
     *    @param whiteWonBgColor bg color if white wins
     *    @param blackWonBgColor bg color if black wins
     *    @param numRows        number of rows (and columns) in the grid
     *    @param game           object that controls the game
     */
    public TicTacToeGUI(String title, JMenu helpMenu, Color usualBgColor,
                         Color whiteWonBgColor, Color blackWonBgColor,
                         int numRows, WhiteBlackGame game)
    {
        super();
        this.usualTitle = title;
        this.usualBgColor = usualBgColor;
        this.whiteWonBgColor = whiteWonBgColor;
        this.blackWonBgColor = blackWonBgColor;
        this.numRows = numRows;
        MinimalFileMenu fileMenu = new MinimalFileMenu();
        fileMenu.addQuitMenuItem();
        includeMenu(fileMenu);
        includeMenu(helpMenu);
        includeStartRestart("Restart", true);
        constructWindowContents(title, usualBgColor, 500, 500,
                                numRows);
        this.game = game;
        restart();
    }

    // Redefines setGrid to also reset the window title and
    // background color of the grid.
    public void setGrid(Grid grid)
    {
        super.setGrid(grid);
        setTitle(usualTitle);
        getDisplay().setBackgroundColor(usualBgColor);
    }

    /** Starts or restarts the application. **/
    public void restart()
    {
        // Create a new grid, initialize a new game, and
        // display it.
        setGrid(new BoundedGrid(true, numRows, numRows));
        game.startGame(getGrid());
        showGrid();
    }

    /** Handles a mouse press over the grid display. **/
    protected void onMousePressOverDisplay(Location loc)
    {
        if ( loc != null )
        {
            game.respondToMousePress(loc);
            if ( game.gameOver() )
            {
                if ( game.whiteWon() )
                    notifyThatWhiteWon();
                else
                    notifyThatBlackWon();
                showGrid();
            }
            else
                getDisplay().updateLocation(loc);
        }
    }

    /** Notifies the players that white won.
     *  Alters the background color of the grid and prints a message to the
     *  console.
     **/
    public void notifyThatWhiteWon()
    {
        setTitle("White won!");
        getDisplay().setBackgroundColor(whiteWonBgColor);
    }

    /** Notifies the players that black won.
     *  Alters the background color of the grid and prints a message to the
     *  console.
     **/
    public void notifyThatBlackWon()
    {
        setTitle("Black won!");
        getDisplay().setBackgroundColor(blackWonBgColor);
    }

}
