//Class: TicTacToeGame
//
// Author: Alyce Brady
//
// Created on May 18, 2004
//
// License Information:
// This class is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation.
//
// This class is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

import java.awt.Color;

import edu.kzoo.grid.Grid;
import edu.kzoo.grid.GridObject;
import edu.kzoo.grid.Location;
import edu.kzoo.grid.TextCell;

/**
 *  Tic-Tac-Toe Game:<br>
 *
 *  A <code>TicTacToeGame</code> object controls the execution of
 *  a game of tic-tac-toe.
 *
 *  @author Alyce Brady
 *  @version May 18, 2004
 **/
public class TicTacToeGame extends WhiteBlackGame
{
    protected static final String WHITE_LABEL = "0";
    protected static final String BLACK_LABEL = "X";
    private String theWinner = null;
 
    /** Constructs a game for 2 players, "O" and "X". **/
    public TicTacToeGame()
    {
        super(new TicTacToePlayer(WHITE_LABEL, Color.WHITE),
              new TicTacToePlayer(BLACK_LABEL, Color.BLACK));
    }

    /** Starts or restarts the application. **/
    public void startGame(Grid grid)
    {
        theWinner = null;
        super.startGame(grid);
    }

    /** Returns true if the game is over; otherwise, returns false. **/
    public boolean gameOver()
    {
        return findWinner() != null;
    }

    /** Returns true if the simulation is over and white won;
     *  false otherwise.
     **/
    public boolean whiteWon()
    {
        String winner = findWinner();
        return winner != null && winner.equals(WHITE_LABEL);
    }

    /** Returns true if the simulation is over black won;
     *  false otherwise.
     **/
    public boolean blackWon()
    {
        String winner = findWinner();
        return winner != null && winner.equals(BLACK_LABEL);
    }

    /** Looks for a winner and returns the label of the winner (or
     *  a null string if no winner found.
     **/
    protected String findWinner()
    {
        if ( theWinner != null )
            return theWinner;

        int invalidRow = getGrid().numRows();
        int invalidHighCol = getGrid().numCols();
        int invalidLowCol = -1;

        // Check for a winner who won with 3 across a row.
        for ( int row = 0; row < getGrid().numRows(); row++ )
            if ( allInARow(row, 0, invalidRow, invalidHighCol, 0, 1) )
                return labelAt(new Location(row, 0));

        // Check for a winner who won with 3 down a column.
        for ( int col = 0; col < getGrid().numCols(); col++ )
            if ( allInARow(0, col, invalidRow, invalidHighCol, 1, 0) )
                return labelAt(new Location(0, col));

        // Check for a winner who won with 3 down the diagonal
        // from the upper-left to the lower-right.
        if ( allInARow(0, 0, invalidRow, invalidHighCol, 1, 1) )
            return labelAt(new Location(0, 0));
 
        // Check for a winner who won with 3 down the diagonal
        // from the upper-right to the lower-left.
        if ( allInARow(0, getGrid().numCols() - 1,
                       invalidRow, invalidLowCol, 1, -1) )
            return labelAt(new Location(0, getGrid().numCols() - 1));

        return null;
    }

    /** Returns true if all of the cells in a row, column, or
     *  diagonal have been filled by the same player; false otherwise.
     **/
    protected boolean allInARow(int firstRow, int firstCol,
                                int beyondLastRow, int beyondLastCol,
                                int rowIncrement, int colIncrement)
    {
        String player = labelAt(new Location(firstRow, firstCol));
        if ( player == null )
            return false;

        for (int row = firstRow + rowIncrement,
                 col = firstCol + colIncrement;
             row != beyondLastRow && col != beyondLastCol;
             row += rowIncrement, col += colIncrement )
        {
            if ( ! player.equals(labelAt(new Location(row, col))) )
                return false;
        }
        return true;
    }

    /** Returns the label of the player at the specified location
     *  (or null if no player has marked the specified location).
     **/
    protected String labelAt(Location loc)
    {
        GridObject obj = getGrid().objectAt(loc);
        if ( obj == null )
            return null;
        else
            return ((TextCell)obj).text();
    }
}