//Class: WhiteBlackGame
//
// Author: Alyce Brady
//
// Created on May 18, 2004
//
// License Information:
// This class is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation.
//
// This class is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

import edu.kzoo.grid.Grid;
import edu.kzoo.grid.Location;

/**
 *  Tic-Tac-Toe Game:<br>
 *
 *  A <code>WhiteBlackGame</code> object controls the execution of
 *  a game that alternates between 2 players called white and black.
 *
 *  @author Alyce Brady
 *  @version May 18, 2004
 **/
public abstract class WhiteBlackGame
{
    private Grid theGrid;
    private Player white;
    private Player black;
    private boolean whitesTurn;

    /** Constructs a game for 2 players, white and black. **/
    public WhiteBlackGame(Player white, Player black)
    {
        this.white = white;
        this.black = black;
    }

    /** Returns the grid in which the game is being played.
     *  Will be null until the <code>startGame</code> method
     *  is invoked.
     **/
    public Grid getGrid()
    {
        return theGrid;
    }

    /** Returns the player represented by white. **/
    public Player whitePlayer()
    {
        return white;
    }

    /** Returns the player represented by black. **/
    public Player blackPlayer()
    {
        return black;
    }

    /** Returns true if it is now white's turn; false otherwise. **/
    public boolean whitesTurn()
    {
        return whitesTurn;
    }

    /** Starts or restarts the game. **/
    public void startGame(Grid grid)
    {
        theGrid = grid;
        whitesTurn = true;
    }

    /** Handles a mouse click on a location of the grid.
     **/
    public void respondToMousePress(Location loc)
    {
        if ( whitesTurn() )
        {
            if ( white.respondToMousePress(theGrid, loc) )
                whitesTurn = false;
        }
        else
            if ( black.respondToMousePress(theGrid, loc) )
                whitesTurn = true;
    }

    /** Returns true if the game is over; otherwise, returns false. **/
    public abstract boolean gameOver();

    /** Returns true if the game is over and white won;
     *  false otherwise.
     **/
    public abstract boolean whiteWon();
}